@@ -48,7 +48,7 @@ writeOGR <- function(obj, dsn, layer, driver, dataset_options=NULL, layer_option
     }
 
     if (is.null(check_exists)) {
-        if (getGDALVersionInfo("VERSION_NUM") >= "1800") {
+        if (as.integer(getGDALVersionInfo("VERSION_NUM")) >= 1800L) {
             check_exists <- TRUE
         } else {
             check_exists <- FALSE
@@ -57,10 +57,10 @@ writeOGR <- function(obj, dsn, layer, driver, dataset_options=NULL, layer_option
 
     if (check_exists) {
         already_exists <- FALSE
-        ogrI <- try(.Call("ogrInfo", as.character(dsn),
-            as.character(layer), PACKAGE = "rgdal"), silent=TRUE)
-        if (class(ogrI) != "try-error") {
-            if (driver == ogrI[[4]]) already_exists <- TRUE
+        ogrI <- .Call("ogrCheckExists", as.character(dsn),
+            as.character(layer), PACKAGE = "rgdal")
+        if (ogrI) {
+            if (driver == attr(ogrI, "driver")) already_exists <- TRUE
         }
         if (already_exists) {
             if (overwrite_layer) {
